#!/usr/bin/env python

import SocketServer, threading, os, socket, StringIO
from hashlib import md5
import numpy as np

from keras.models import Sequential
from keras.layers import ZeroPadding2D
from keras.layers.core import Dense, Dropout, Flatten
from keras.layers.convolutional import Convolution2D, MaxPooling2D

m = Sequential()

width = 320
height = 240

m.add(ZeroPadding2D((1,1),input_shape=(3,height,width)))
m.add(Convolution2D(64, 6, 6, activation='relu'))
m.add(ZeroPadding2D((1,1)))
m.add(Convolution2D(64, 6, 6, activation='relu'))
m.add(MaxPooling2D((4,4), strides=(4,4)))

m.add(Convolution2D(64, 4, 4, activation='relu'))
m.add(ZeroPadding2D((1,1)))
m.add(Convolution2D(64, 4, 4, activation='relu'))
m.add(MaxPooling2D((3,3), strides=(3,3)))

m.add(Convolution2D(128, 3, 3, activation='relu'))
m.add(ZeroPadding2D((1,1)))
m.add(Convolution2D(128, 3, 3, activation='relu'))
m.add(MaxPooling2D((3,3), strides=(3,3)))

m.add(Convolution2D(192, 3, 3, activation='relu'))
m.add(ZeroPadding2D((1,1)))
m.add(Convolution2D(192, 3, 3, activation='relu'))
m.add(MaxPooling2D((3,3), strides=(3,3)))

m.add(Flatten())
m.add(Dropout(0.3))
m.add(Dense(256, activation='relu'))
m.add(Dropout(0.4))
m.add(Dense(256, activation='relu'))
m.add(Dropout(0.4))
m.add(Dense(2, activation='softmax'))

m.compile(optimizer='sgd', loss='binary_crossentropy')

m.load_weights("model")

def is_good_image(image):
  bad,good = list(m.predict(np.array([image]))[0])
  print good
  return good > 0.99

#### end of image stuff, now server stuff ####
def recvall(sock, n):
  buf = ''
  while len(buf) < n:
    tmp = sock.recv(1)
    if not tmp:
      return None
    buf += tmp
  return buf

class threadedserver(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
  pass

class incoming(SocketServer.BaseRequestHandler):
  def handle(self):
    salt = os.urandom(6).encode("hex")
    self.request.send(salt)
    response = recvall(self.request, 6)

    if not md5(salt+response).digest().startswith("\x00\x00\x00"):
      return

    self.request.settimeout(2)
    #read each of the 320x240 pixels, in red, green, and blue 320x240x3
    red   = np.frombuffer(recvall(self.request, 4*320*240), dtype='float32').reshape(240,320)
    green = np.frombuffer(recvall(self.request, 4*320*240), dtype='float32').reshape(240,320)
    blue  = np.frombuffer(recvall(self.request, 4*320*240), dtype='float32').reshape(240,320)
    image = np.array([red, green, blue])
    if is_good_image(image):
      self.request.send(open("flag.txt").read())
    else:
      self.request.send("BAD\n")
    
    

if __name__ == "__main__":
  SocketServer.TCPServer.allow_reuse_address = True
  server = threadedserver(("0.0.0.0", 29281), incoming)
  server.timeout = 4
  server_thread = threading.Thread(target=server.serve_forever)
  server_thread.daemon = False
  server_thread.start()
